@@ -1,8 +1,8 @@
-#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#   ---------------------------------------------------------------------- copyright and license ---
 #
 #   file: lib/Dist/Zilla/Plugin/Manifest/Read.pm
 #
-#   Copyright © 2015 Van de Bugger
+#   Copyright © 2015, 2016 Van de Bugger.
 #
 #   This file is part of perl-Dist-Zilla-Plugin-Manifest-Read.
 #
@@ -17,7 +17,7 @@
 #   You should have received a copy of the GNU General Public License along with
 #   perl-Dist-Zilla-Plugin-Manifest-Read. If not, see <http://www.gnu.org/licenses/>.
 #
-#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#   ---------------------------------------------------------------------- copyright and license ---
 
 #pod =for :this This is C<Dist::Zilla::Plugin::Manifest::Read> module documentation. Read this if you are going to hack or
 #pod extend C<Manifest::Read>, or use it programmatically.
@@ -58,7 +58,7 @@ use namespace::autoclean;
 use version 0.77;
 
 # ABSTRACT: Read annotated source manifest
-our $VERSION = 'v0.4.2_02'; # TRIAL VERSION
+our $VERSION = 'v0.4.2_03'; # TRIAL VERSION
 
 with 'Dist::Zilla::Role::FileGatherer';
 with 'Dist::Zilla::Role::FileFinder';
@@ -67,6 +67,7 @@ with 'Dist::Zilla::Role::ErrorLogger' => { -version => 0.006 }; # need log_error
 use Dist::Zilla::File::OnDisk;
 use List::Util qw{ min max };
 use Path::Tiny;
+use Set::Object qw{ set };
 use Try::Tiny;
 
 # --------------------------------------------------------------------------------------------------
@@ -124,7 +125,7 @@ sub _build_manifest_file {
 
 #pod =method BUILD
 #pod
-#pod This method creates bunch of file finders: C<Manifest::Read/:AllFiles>, C<Manifest::Read/:ExecFiles>, C<Manifest::Read/:ExtraTestFiles>, C<Manifest::Read/:InstallModules>, C<Manifest::Read/:PerlExecFiles>, C<Manifest::Read/:TestFiles>.
+#pod This method creates bunch of file finders: C<Manifest::Read/:AllFiles>, C<Manifest::Read/:ExecFiles>, C<Manifest::Read/:ExtraTestFiles>, C<Manifest::Read/:IncModules>, C<Manifest::Read/:InstallModules>, C<Manifest::Read/:NoFiles>, C<Manifest::Read/:PerlExecFiles>, C<Manifest::Read/:ShareFiles>, C<Manifest::Read/:TestFiles>.
 #pod
 #pod =cut
 
@@ -133,7 +134,7 @@ sub BUILD {
     require Dist::Zilla::Plugin::FinderCode;
     my $parent = $self;
     my $zilla  = $self->zilla;
-    my @finders = qw{ :AllFiles :ExecFiles :ExtraTestFiles :InstallModules :PerlExecFiles :TestFiles };
+    my @finders = qw{ :AllFiles :ExecFiles :ExtraTestFiles :IncModules :InstallModules :NoFiles :PerlExecFiles :ShareFiles :TestFiles };
     for my $name ( @finders ) {
         my $finder = Dist::Zilla::Plugin::FinderCode->new( {
             plugin_name => $parent->plugin_name . '/' . $name,
@@ -145,15 +146,9 @@ sub BUILD {
                 if ( not defined( $plugin ) ) {
                     $parent->abort( [ "Can't find plugin %s", $name ] );
                 };
-                my $m = $parent->_manifest_bulk;
-                my $files = [];
-                for my $file ( @{ $plugin->find_files() } ) {
-                    my $item = $m->{ $file->name } or next;
-                    $item->{ file } == $file       or next;
-                    $item->{ mark } ne '-'         or next;
-                    push( @$files, $file );
-                };
-                return $files;
+                my $files = $parent->_incl_file_set * set( @{ $plugin->find_files() // [] } );
+                    # `:NoFiles` in `Dist::Zilla` 6.008 returns `undef`, not `[]`.  ^^^^^
+                return [ $files->members ];
             },
         } );
         push( @{ $zilla->plugins }, $finder );
@@ -172,7 +167,7 @@ sub BUILD {
 
 sub gather_files {
     my ( $self ) = @_;
-    for my $file ( @{ $self->_incl_file_list } ) {
+    for my $file ( $self->_incl_file_set->members ) {
         $self->add_file( $file );
     };
     return;
@@ -195,7 +190,14 @@ sub gather_files {
 #pod
 #pod =cut
 
-sub find_files {
+has _complete_file_list => (
+    isa         => 'ArrayRef',
+    is          => 'ro',
+    lazy        => 1,
+    builder     => '_build_complete_file_list',
+);
+
+sub _build_complete_file_list {
     my ( $self ) = @_;
     return [
         map(
@@ -206,14 +208,42 @@ sub find_files {
             )
         )
     ];
-    #~ return [ @{ $self->_incl_file_list } ];
+};
+
+sub find_files {
+    my ( $self ) = @_;
+    return [ @{ $self->_complete_file_list } ];
+};
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =attr _incl_file_set
+#pod
+#pod Set of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest I<and> marked
+#pod for inclusion to the distribution.
+#pod
+#pod =cut
+
+has _incl_file_set => (
+    isa         => 'Set::Object',
+    is          => 'ro',
+    lazy        => 1,
+    builder     => '_build_incl_file_set',
+    init_arg    => undef,
+);
+
+sub _build_incl_file_set {
+    my ( $self ) = @_;
+    my $m = $self->_manifest_bulk;
+    return set( map( { $_->{ file } } grep( { $_->{ mark } ne '-' } values( %$m ) ) ));
 };
 
 # --------------------------------------------------------------------------------------------------
 
 #pod =attr _manifest_bulk
 #pod
-#pod Parsed manifest. Hash of hashes. Each inner hash has keys and associated values:
+#pod Parsed manifest. HashRef. Keys are file names, values are HashRefs to inner hashes. Each inner hash
+#pod has keys and associated values:
 #pod
 #pod =for :list
 #pod =   name
@@ -271,21 +301,6 @@ sub _build_manifest_bulk {
 
 # --------------------------------------------------------------------------------------------------
 
-#pod =method _incl_file_list
-#pod
-#pod Returns array of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest
-#pod I<and> marked for inclusion to the distribution.
-#pod
-#pod =cut
-
-sub _incl_file_list {
-    my ( $self ) = @_;
-    my $m = $self->_manifest_bulk;
-    return [ map( { $m->{ $_ }->{ file } } grep( { $m->{ $_ }->{ mark } ne '-' } keys( %$m ) ) ) ];
-};
-
-# --------------------------------------------------------------------------------------------------
-
 #pod =attr _manifest_lines
 #pod
 #pod Array of chomped manifest lines, including comments and empty lines.
@@ -447,7 +462,7 @@ __PACKAGE__->meta->make_immutable;
 #pod
 #pod =head1 COPYRIGHT AND LICENSE
 #pod
-#pod Copyright (C) 2015 Van de Bugger
+#pod Copyright (C) 2015, 2016 Van de Bugger
 #pod
 #pod License GPLv3+: The GNU General Public License version 3 or later
 #pod <http://www.gnu.org/licenses/gpl-3.0.txt>.
@@ -472,7 +487,7 @@ Dist::Zilla::Plugin::Manifest::Read - Read annotated source manifest
 
 =head1 VERSION
 
-Version v0.4.2_02, released on 2016-11-04 23:54 UTC.
+Version v0.4.2_03, released on 2016-11-13 15:04 UTC.
 This is a B<trial release>.
 
 =head1 WHAT?
@@ -524,9 +539,15 @@ Manifest file as a C<Dist::Zilla> file object (C<Dist::Zilla::File::OnDisk>).
 
 C<Object>, read-only.
 
+=head2 _incl_file_set
+
+Set of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest I<and> marked
+for inclusion to the distribution.
+
 =head2 _manifest_bulk
 
-Parsed manifest. Hash of hashes. Each inner hash has keys and associated values:
+Parsed manifest. HashRef. Keys are file names, values are HashRefs to inner hashes. Each inner hash
+has keys and associated values:
 
 =over 4
 
@@ -564,7 +585,7 @@ C<ArrayRef[Str]>, read-only, lazy, initialized with builder.
 
 =head2 BUILD
 
-This method creates bunch of file finders: C<Manifest::Read/:AllFiles>, C<Manifest::Read/:ExecFiles>, C<Manifest::Read/:ExtraTestFiles>, C<Manifest::Read/:InstallModules>, C<Manifest::Read/:PerlExecFiles>, C<Manifest::Read/:TestFiles>.
+This method creates bunch of file finders: C<Manifest::Read/:AllFiles>, C<Manifest::Read/:ExecFiles>, C<Manifest::Read/:ExtraTestFiles>, C<Manifest::Read/:IncModules>, C<Manifest::Read/:InstallModules>, C<Manifest::Read/:NoFiles>, C<Manifest::Read/:PerlExecFiles>, C<Manifest::Read/:ShareFiles>, C<Manifest::Read/:TestFiles>.
 
 =head2 gather_files
 
@@ -584,11 +605,6 @@ the method.
 
 If you are interested in distribution files, have look to file finders generated by C<BUILD>.
 
-=head2 _incl_file_list
-
-Returns array of files (object which do C<Dist::Zilla::Role::File> role) listed in the manifest
-I<and> marked for inclusion to the distribution.
-
 =head2 _parse_lines
 
 This method parses manifest lines. Each line is parsed separately (there is no line continuation).
@@ -637,7 +653,7 @@ Van de Bugger <van.de.bugger@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2015 Van de Bugger
+Copyright (C) 2015, 2016 Van de Bugger
 
 License GPLv3+: The GNU General Public License version 3 or later
 <http://www.gnu.org/licenses/gpl-3.0.txt>.
